@@ -96,9 +96,9 @@ def test_stats():
     # ---- custom stats ----
     # expected results
     custom_stats_results = {
-        'zone':       [1, 2, 3],
-        'double_sum': [12, 16, 24],
-        'range':      [0,   0,  0],
+        'zone':       [1, 2],
+        'double_sum': [12, 16],
+        'range':      [0,   0],
     }
 
     def _double_sum(values):
@@ -115,12 +115,12 @@ def test_stats():
     # numpy case
     df_np = stats(
         zones=zones_np, values=values_np, stats_funcs=custom_stats,
-        nodata_zones=0, nodata_values=0
+        zone_ids=[1, 2], nodata_zones=0, nodata_values=0
     )
     # dask case
     df_da = stats(
         zones=zones_da, values=values_da, stats_funcs=custom_stats,
-        nodata_zones=0, nodata_values=0
+        zone_ids=[1, 2], nodata_zones=0, nodata_values=0
     )
     check_results(df_np, df_da, custom_stats_results)
 
@@ -149,20 +149,19 @@ def test_crosstab_2d():
     # percentage agg, expected results
 
     crosstab_2d_percentage_results = {
-        'zone': [1,   2,  3],
-        0:      [0,   0,  100],
-        1:      [100, 0,  0],
-        2:      [0,   100, 0],
+        'zone': [1,   2],
+        1:      [100, 0],
+        2:      [0,   100],
     }
 
     # numpy case
     df_np = crosstab(
-        zones=zones_np, values=values_np,
+        zones=zones_np, values=values_np, zone_ids=[1, 2], cat_ids=[1, 2],
         nodata_zones=0, nodata_values=3, agg='percentage'
     )
     # dask case
     df_da = crosstab(
-        zones=zones_da, values=values_da,
+        zones=zones_da, values=values_da, zone_ids=[1, 2], cat_ids=[1, 2],
         nodata_zones=0, nodata_values=3, agg='percentage'
     )
     check_results(df_np, df_da, crosstab_2d_percentage_results)
@@ -171,22 +170,23 @@ def test_crosstab_2d():
 def test_crosstab_3d():
     # expected results
     crosstab_3d_results = {
-        'zone': [0, 1, 2, 3],
-        'cat1': [6, 6, 5, 6],
-        'cat2': [6, 6, 5, 6],
-        'cat3': [6, 6, 5, 6],
-        'cat4': [6, 6, 5, 6],
+        'zone': [1, 2, 3],
+        'cat1': [6, 5, 6],
+        'cat2': [6, 5, 6],
+        'cat3': [6, 5, 6],
+        'cat4': [6, 5, 6],
     }
 
     # numpy case
     zones_np, _, values_np = create_zones_values(backend='numpy')
     df_np = crosstab(
-        zones=zones_np, values=values_np, layer=-1
+        zones=zones_np, values=values_np, zone_ids=[1, 2, 3], layer=-1
     )
     # dask case
     zones_da, _, values_da = create_zones_values(backend='dask')
     df_da = crosstab(
-        zones=zones_da, values=values_da, layer=-1
+        zones=zones_da, values=values_da, zone_ids=[1, 2, 3],
+        cat_ids=['cat1', 'cat2', 'cat3', 'cat4'], layer=-1
     )
     check_results(df_np, df_da, crosstab_3d_results)
 
